#
# --------------------------------------------------------------------------
#  Gurux Ltd
#
#
#
# Filename:        $HeadURL$
#
# Version:         $Revision$,
#                  $Date$
#                  $Author$
#
# Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#  DESCRIPTION
#
# This file is a part of Gurux Device Framework.
#
# Gurux Device Framework is Open Source software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2 of the License.
# Gurux Device Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# More information of Gurux products: https:#www.gurux.org
#
# This code is licensed under the GNU General Public License v2.
# Full text may be retrieved at http:#www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from os import listdir
from os.path import isfile, join
from .enums.KeyUsage import KeyUsage
from .GXAsn1Converter import GXAsn1Converter
from .GXx509Certificate import GXx509Certificate


class GXx509CertificateCollection(list):
    """
    List of x509 certificates.
    """

    def find(self, key):
        """
        Find public key certificate by public key.

            Parameters:
                key: X509 certificate to search for.

            Returns:
                Certificate found or null, if the certificate is not found.
        """
        for it in self:
            if it.publicKey == key:
                return it
        return None

    def findBySerial(self, serialNumber, issuer):
        """
        Find public key certificate by serial number.

            Parameters:
                serialNumber: X509 certificate serial number to search for.
                issuer: X509 certificate issuer.

            Returns:
                Certificate found or null, if the certificate is not found.
        """
        for it in self:
            if it.serialNumber == serialNumber and it.issuer == issuer:
                return it
        return None

    def findBySystemTitle(self, systemTitle, usage):
        """
        Find public key certificate by system title.

            Parameters:
                systemTitle: System title.
                usage: Key usage.

            Returns:
                Certificate found or null, if the certificate is not found.
        """
        if not systemTitle:
            commonName = None
        else:
            commonName = GXAsn1Converter.systemTitleToSubject(systemTitle)
        return self.findByCommonName(commonName, usage)

    def getCertificates(self, usage):
        """
        Find certificates by key usage.

            Parameters:
                usage: Key usage.

            Returns:
                Found certificates.
        """
        certificates = []
        for it in self:
            if it.keyUsage == usage:
                certificates.append(it)
        return certificates

    def findByCommonName(self, commonName, usage):
        """
        Find public key certificate by common name (CN).

            Parameters:
                commonName: Common name.
                usage: Key usage.

            Returns:
                Certificate found or null, if the certificate is not found.
        """
        for it in self:
            if (
                usage == KeyUsage.NONE
                or (it.keyUsage and usage) != 0
                and it.subject.contains(commonName)
            ):
                return it
        return None

    def import_(self, folder):
        """
        Import certificates from the given folder.

            Parameters:
                path: The folder from which the certificates are imported.
        """

        files = [f for f in listdir(folder) if isfile(join(folder, f))]
        for it in files:
            if it.endswith(".pem") or it.endswith(".cer"):
                cert = GXx509Certificate.load(it)
                self.append(cert)



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http:#www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http:#www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from gurux_dlms.asn.enums.HashAlgorithm import HashAlgorithm


class HashAlgorithmConverter:
    @classmethod
    def getString(cls, value):
        if value == HashAlgorithm.SHA1RSA:
            return "1.2.840.113549.1.1.5"
        elif value == HashAlgorithm.MD5RSA:
            return "1.2.840.113549.1.1.4"
        elif value == HashAlgorithm.SHA1DSA:
            return "1.2.840.10040.4.3"
        elif value == HashAlgorithm.SHA1RSA1:
            return "1.3.14.3.2.29"
        elif value == HashAlgorithm.SHA_RSA:
            return "1.3.14.3.2.15"
        elif value == HashAlgorithm.MD5RSA1:
            return "1.3.14.3.2.3"
        elif value == HashAlgorithm.MD2RSA1:
            return "1.2.840.113549.1.1.2"
        elif value == HashAlgorithm.MD4RSA:
            return "1.2.840.113549.1.1.3"
        elif value == HashAlgorithm.MD4RSA1:
            return "1.3.14.3.2.2"
        elif value == HashAlgorithm.MD4RSA2:
            return "1.3.14.3.2.4"
        elif value == HashAlgorithm.MD2RSA:
            return "1.3.14.7.2.3.1"
        elif value == HashAlgorithm.SHA1DSA1:
            return "1.3.14.3.2.13"
        elif value == HashAlgorithm.DSA_SHA1:
            return "1.3.14.3.2.27"
        elif value == HashAlgorithm.MOSAIC_UPDATED_SIG:
            return "2.16.840.1.101.2.1.1.19"
        elif value == HashAlgorithm.SHA1NO_SIGN:
            return "1.3.14.3.2.26"
        elif value == HashAlgorithm.MD5NO_SIGN:
            return "1.2.840.113549.2.5"
        elif value == HashAlgorithm.SHA256NO_SIGN:
            return "2.16.840.1.101.3.4.2.1"
        elif value == HashAlgorithm.SHA384NO_SIGN:
            return "2.16.840.1.101.3.4.2.2"
        elif value == HashAlgorithm.SHA512NO_SIGN:
            return "2.16.840.1.101.3.4.2.3"
        elif value == HashAlgorithm.SHA256RSA:
            return "1.2.840.113549.1.1.11"
        elif value == HashAlgorithm.SHA384RSA:
            return "1.2.840.113549.1.1.12"
        elif value == HashAlgorithm.SHA512RSA:
            return "1.2.840.113549.1.1.13"
        elif value == HashAlgorithm.RSA_SSA_PSS:
            return "1.2.840.113549.1.1.10"
        elif value == HashAlgorithm.SHA1WITHECDSA:
            return "1.2.840.10045.4.1"
        elif value == HashAlgorithm.SHA256WITH_ECDSA:
            return "1.2.840.10045.4.3.2"
        elif value == HashAlgorithm.SHA384WITH_ECDSA:
            return "1.2.840.10045.4.3.3"
        elif value == HashAlgorithm.SHA512WITH_ECDSA:
            return "1.2.840.10045.4.3.4"
        elif value == HashAlgorithm.SPECIFIED_ECDSA:
            return "1.2.840.10045.4.3"
        else:
            raise ValueError("Invalid HashAlgorithm. " + value)

    @classmethod
    def fromString(cls, value):
        if value == "1.2.840.113549.1.1.5":
            return HashAlgorithm.SHA1RSA
        elif value == "1.2.840.113549.1.1.4":
            return HashAlgorithm.MD5RSA
        elif value == "1.2.840.10040.4.3":
            return HashAlgorithm.SHA1DSA
        elif value == "1.3.14.3.2.29":
            return HashAlgorithm.SHA1RSA1
        elif value == "1.3.14.3.2.15":
            return HashAlgorithm.SHA_RSA
        elif value == "1.3.14.3.2.3":
            return HashAlgorithm.MD5RSA1
        elif value == "1.2.840.113549.1.1.2":
            return HashAlgorithm.MD2RSA1
        elif value == "1.2.840.113549.1.1.3":
            return HashAlgorithm.MD4RSA
        elif value == "1.3.14.3.2.2":
            return HashAlgorithm.MD4RSA1
        elif value == "1.3.14.3.2.4":
            return HashAlgorithm.MD4RSA2
        elif value == "1.3.14.7.2.3.1":
            return HashAlgorithm.MD2RSA
        elif value == "1.3.14.3.2.13":
            return HashAlgorithm.SHA1DSA1
        elif value == "1.3.14.3.2.27":
            return HashAlgorithm.DSA_SHA1
        elif value == "2.16.840.1.101.2.1.1.19":
            return HashAlgorithm.MOSAIC_UPDATED_SIG
        elif value == "1.3.14.3.2.26":
            return HashAlgorithm.SHA1NO_SIGN
        elif value == "1.2.840.113549.2.5":
            return HashAlgorithm.MD5NO_SIGN
        elif value == "2.16.840.1.101.3.4.2.1":
            return HashAlgorithm.SHA256NO_SIGN
        elif value == "2.16.840.1.101.3.4.2.2":
            return HashAlgorithm.SHA384NO_SIGN
        elif value == "2.16.840.1.101.3.4.2.3":
            return HashAlgorithm.SHA512NO_SIGN
        elif value == "1.2.840.113549.1.1.11":
            return HashAlgorithm.SHA256RSA
        elif value == "1.2.840.113549.1.1.12":
            return HashAlgorithm.SHA384RSA
        elif value == "1.2.840.113549.1.1.13":
            return HashAlgorithm.SHA512RSA
        elif value == "1.2.840.113549.1.1.10":
            return HashAlgorithm.RSA_SSA_PSS
        elif value == "1.2.840.10045.4.1":
            return HashAlgorithm.SHA1WITHECDSA
        elif value == "1.2.840.10045.4.3.2":
            return HashAlgorithm.SHA256WITH_ECDSA
        elif value == "1.2.840.10045.4.3.3":
            return HashAlgorithm.SHA384WITH_ECDSA
        elif value == "1.2.840.10045.4.3.4":
            return HashAlgorithm.SHA512WITH_ECDSA
        elif value == "1.2.840.10045.4.3":
            return HashAlgorithm.SPECIFIED_ECDSA
        else:
            return HashAlgorithm.NONE



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http:#www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http:#www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
class GXDLMSBroadcastLogTable:
    '''
    Broadcast log table
    '''

    __sourceAddress = None
    "Source address of a broadcast packet."

    __sequenceNumber = 0
    "The sequence number contained in the BC0 header"

    __validTime = None
    "Remaining time in minutes until when this entry in the broadcast log table is considered valid."

    @property
    def sourceAddress(self):
        '''
        Source address of a broadcast packet.
        '''
        return self.__sourceAddress

    @sourceAddress.setter
    def sourceAddress(self, value):
        self.__sourceAddress = value
    @property
    def sequenceNumber(self):
        '''
        The sequence number contained in the BC0 header
        '''
        return self.__sequenceNumber

    @sequenceNumber.setter
    def sequenceNumber(self, value):
        self.__sequenceNumber = value
    @property
    def validTime(self):
        '''
        Remaining time in minutes until when this entry in the broadcast log table is considered valid.
        '''
        return self.__validTime

    @validTime.setter
    def validTime(self, value):
        self.__validTime = value



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from ..enums import DataType
from ..enums import AccessMode
from ..enums import MethodAccessMode
from ..enums import AccessMode3
from ..enums import MethodAccessMode3


###Python 2 requires this
# pylint: disable=bad-option-value,old-style-class,too-few-public-methods, too-many-instance-attributes
class GXDLMSAttributeSettings:
    def __init__(self, index=0):
        #
        # Constructor.
        # index: Attribute index.
        #
        self.name = ""
        self.index = index
        self.type_ = DataType.NONE
        self.uiType = DataType.NONE
        self.access = AccessMode.READ_WRITE
        self.access3 = AccessMode3.READ_WRITE
        self.methodAccess = MethodAccessMode.ACCESS
        self.methodAccess3 = MethodAccessMode3.ACCESS
        self.static = False
        self.values = None
        self.order = 0
        self.minimumVersion = 0
        self.xml = ""

    def copyTo(self, target):
        target.name = self.name
        target.index = self.index
        target.type_ = self.type_
        target.uiType = self.uiType
        target.access = self.access
        target.access3 = self.access3
        target.methodAccess = self.methodAccess
        target.methodAccess3 = self.methodAccess3
        target.static = self.static
        target.values = self.values
        target.order = self.order
        target.minimumVersion = self.minimumVersion
        target.xml = self.xml



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                $Date$
#                $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import abc

ABC = abc.ABCMeta("ABC", (object,), {"__slots__": ()})

class IGXCryptoNotifier(ABC):
    """
    The server uses this interface to notify client connections.
    """

    __metaclass__ = abc.ABCMeta
    """Network Server component will notify events throught this interface."""

    @abc.abstractmethod
    def onPduEventHandler(self, sender, complete, data):
        """Notifies un-ciphered PDU..

        sender : The source of the event.
        complete : Is all data received.
        data : Un-ciphered PDU.
        """

    @abc.abstractmethod
    def onKey(self, sender, args):
        """Called when the public or private key is needed 
        and it's unknown.

        sender : The source of the event.
        args : Event arguments.
        """

    @abc.abstractmethod
    def onCrypto(self, sender, args):
        """Called to encrypt or decrypt the data using 
        external Hardware Security Module.

        sender : The source of the event.
        args : Event arguments.
        """
